@@ -30,28 +30,6 @@
 
 package com.twelvemonkeys.imageio.plugins.bmp;
 
-import java.awt.*;
-import java.awt.color.ColorSpace;
-import java.awt.image.*;
-import java.io.DataInput;
-import java.io.File;
-import java.io.IOException;
-import java.nio.ByteOrder;
-import java.util.Arrays;
-import java.util.Iterator;
-
-import javax.imageio.IIOException;
-import javax.imageio.ImageIO;
-import javax.imageio.ImageReadParam;
-import javax.imageio.ImageReader;
-import javax.imageio.ImageTypeSpecifier;
-import javax.imageio.event.IIOReadUpdateListener;
-import javax.imageio.event.IIOReadWarningListener;
-import javax.imageio.metadata.IIOMetadata;
-import javax.imageio.metadata.IIOMetadataFormatImpl;
-import javax.imageio.spi.ImageReaderSpi;
-import javax.imageio.stream.ImageInputStream;
-
 import com.twelvemonkeys.imageio.ImageReaderBase;
 import com.twelvemonkeys.imageio.stream.SubImageInputStream;
 import com.twelvemonkeys.imageio.util.IIOUtil;
@@ -61,6 +39,23 @@ import com.twelvemonkeys.io.LittleEndianDataInputStream;
 import com.twelvemonkeys.io.enc.DecoderStream;
 import com.twelvemonkeys.xml.XMLSerializer;
 
+import javax.imageio.*;
+import javax.imageio.event.IIOReadUpdateListener;
+import javax.imageio.event.IIOReadWarningListener;
+import javax.imageio.metadata.IIOMetadata;
+import javax.imageio.metadata.IIOMetadataFormatImpl;
+import javax.imageio.spi.ImageReaderSpi;
+import javax.imageio.stream.ImageInputStream;
+import java.awt.*;
+import java.awt.color.ColorSpace;
+import java.awt.image.*;
+import java.io.DataInput;
+import java.io.File;
+import java.io.IOException;
+import java.nio.ByteOrder;
+import java.util.Collections;
+import java.util.Iterator;
+
 /**
  * ImageReader for Microsoft Windows Bitmap (BMP) format.
  *
@@ -205,7 +200,7 @@ public final class BMPImageReader extends ImageReaderBase {
         checkBounds(pImageIndex);
 
         // TODO: Better implementation, include INT_RGB types for 3BYTE_BGR and 4BYTE_ABGR for INT_ARGB
-        return Arrays.asList(getRawImageType(pImageIndex)).iterator();
+        return Collections.singletonList(getRawImageType(pImageIndex)).iterator();
     }
 
     @Override
@@ -410,6 +405,13 @@ public final class BMPImageReader extends ImageReaderBase {
 
     private ImageReader initReaderDelegate(int compression) throws IOException {
         ImageReader reader = getImageReaderDelegate(compression);
+        reader.reset();
+
+        // Install listener
+        ListenerDelegator listenerDelegator = new ListenerDelegator();
+        reader.addIIOReadWarningListener(listenerDelegator);
+        reader.addIIOReadProgressListener(listenerDelegator);
+        reader.addIIOReadUpdateListener(listenerDelegator);
 
         imageInput.seek(pixelOffset);
         reader.setInput(new SubImageInputStream(imageInput, header.getImageSize()));
@@ -450,12 +452,6 @@ public final class BMPImageReader extends ImageReaderBase {
 
         ImageReader reader = readers.next();
 
-        // Install listener
-        ListenerDelegator listenerDelegator = new ListenerDelegator();
-        reader.addIIOReadWarningListener(listenerDelegator);
-        reader.addIIOReadProgressListener(listenerDelegator);
-        reader.addIIOReadUpdateListener(listenerDelegator);
-
         // Cache for later use
         switch (compression) {
             case DIB.COMPRESSION_JPEG:
@@ -481,8 +477,12 @@ public final class BMPImageReader extends ImageReaderBase {
 
     private void readRowByte(final DataInput input, final int height, final Rectangle srcRegion, final int xSub, final int ySub,
                              final byte[] rowDataByte, final WritableRaster destChannel, final Raster srcChannel, final int y) throws IOException {
+        // Flip into position?
+        int srcY = !header.topDown ? height - 1 - y : y;
+        int dstY = (srcY - srcRegion.y) / ySub;
+
         // If subsampled or outside source region, skip entire row
-        if (y % ySub != 0 || height - 1 - y < srcRegion.y || height - 1 - y >= srcRegion.y + srcRegion.height) {
+        if (srcY % ySub != 0 || srcY < srcRegion.y || srcY >= srcRegion.y + srcRegion.height) {
             input.skipBytes(rowDataByte.length);
 
             return;
@@ -497,19 +497,17 @@ public final class BMPImageReader extends ImageReaderBase {
             }
         }
 
-        if (header.topDown) {
-            destChannel.setDataElements(0, y, srcChannel);
-        } else {
-            // Flip into position
-            int dstY = (height - 1 - y - srcRegion.y) / ySub;
-            destChannel.setDataElements(0, dstY, srcChannel);
-        }
+        destChannel.setDataElements(0, dstY, srcChannel);
     }
 
     private void readRowUShort(final DataInput input, final int height, final Rectangle srcRegion, final int xSub, final int ySub,
                                final short[] rowDataUShort, final WritableRaster destChannel, final Raster srcChannel, final int y) throws IOException {
+        // Flip into position?
+        int srcY = !header.topDown ? height - 1 - y : y;
+        int dstY = (srcY - srcRegion.y) / ySub;
+
         // If subsampled or outside source region, skip entire row
-        if (y % ySub != 0 || height - 1 - y < srcRegion.y || height - 1 - y >= srcRegion.y + srcRegion.height) {
+        if (srcY % ySub != 0 || srcY < srcRegion.y || srcY >= srcRegion.y + srcRegion.height) {
             input.skipBytes(rowDataUShort.length * 2 + (rowDataUShort.length % 2) * 2);
 
             return;
@@ -529,19 +527,17 @@ public final class BMPImageReader extends ImageReaderBase {
             }
         }
 
-        if (header.topDown) {
-            destChannel.setDataElements(0, y, srcChannel);
-        } else {
-            // Flip into position
-            int dstY = (height - 1 - y - srcRegion.y) / ySub;
-            destChannel.setDataElements(0, dstY, srcChannel);
-        }
+        destChannel.setDataElements(0, dstY, srcChannel);
     }
 
     private void readRowInt(final DataInput input, final int height, final Rectangle srcRegion, final int xSub, final int ySub,
                             final int[] rowDataInt, final WritableRaster destChannel, final Raster srcChannel, final int y) throws IOException {
+        // Flip into position?
+        int srcY = !header.topDown ? height - 1 - y : y;
+        int dstY = (srcY - srcRegion.y) / ySub;
+
         // If subsampled or outside source region, skip entire row
-        if (y % ySub != 0 || height - 1 - y < srcRegion.y || height - 1 - y >= srcRegion.y + srcRegion.height) {
+        if (srcY % ySub != 0 || srcY < srcRegion.y || srcY >= srcRegion.y + srcRegion.height) {
             input.skipBytes(rowDataInt.length * 4);
 
             return;
@@ -556,13 +552,7 @@ public final class BMPImageReader extends ImageReaderBase {
             }
         }
 
-        if (header.topDown) {
-            destChannel.setDataElements(0, y, srcChannel);
-        } else {
-            // Flip into position
-            int dstY = (height - 1 - y - srcRegion.y) / ySub;
-            destChannel.setDataElements(0, dstY, srcChannel);
-        }
+        destChannel.setDataElements(0, dstY, srcChannel);
     }
 
     // TODO: Candidate util method
@@ -633,7 +623,8 @@ public final class BMPImageReader extends ImageReaderBase {
         return new BMPMetadata(header, colors);
     }
 
-    public static void main(String[] args) throws IOException {
+    @SuppressWarnings("ConstantConditions")
+    public static void main(String[] args) {
         BMPImageReaderSpi provider = new BMPImageReaderSpi();
         BMPImageReader reader = new BMPImageReader(provider);
 
@@ -686,7 +677,7 @@ public final class BMPImageReader extends ImageReaderBase {
         }
     }
 
-    @SuppressWarnings({"unchecked", "UnusedDeclaration"})
+    @SuppressWarnings({ "unchecked", "UnusedDeclaration", "SameParameterValue" })
     static <T extends Throwable> void throwAs(final Class<T> pType, final Throwable pThrowable) throws T {
         throw (T) pThrowable;
     }
