@@ -36,7 +36,7 @@ import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
-import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
 
 final class Plain16BitDecoder extends InputStream {
     private final BufferedReader reader;
@@ -45,7 +45,7 @@ final class Plain16BitDecoder extends InputStream {
     private int leftOver = -1;
 
     public Plain16BitDecoder(final InputStream in) {
-        reader = new BufferedReader(new InputStreamReader(in, Charset.forName("ASCII")));
+        reader = new BufferedReader(new InputStreamReader(in, StandardCharsets.US_ASCII));
     }
 
     @Override
