@@ -58,8 +58,6 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
     private final boolean optionUncompressed;
     private final boolean optionByteAligned;
 
-    // Need to take fill order into account (?) (use flip table?)
-    private final int fillOrder;
     private final int type;
 
     private int decodedLength;
@@ -81,12 +79,10 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
      * @param columns the number of columns in the stream.
      * @param type the type of stream, must be one of {@code COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE},
      *             {@code COMPRESSION_CCITT_T4} or {@code COMPRESSION_CCITT_T6}.
-     * @param fillOrder fillOrder, must be {@code FILL_LEFT_TO_RIGHT} or
-     * {@code FILL_RIGHT_TO_LEFT}.
      * @param options CCITT T.4 or T.6 options.
      * @param byteAligned enable byte alignment used in PDF files (EncodedByteAlign).
      */
-    public CCITTFaxDecoderStream(final InputStream stream, final int columns, final int type, final int fillOrder,
+    public CCITTFaxDecoderStream(final InputStream stream, final int columns, final int type,
                                  final long options, final boolean byteAligned) {
         super(Validate.notNull(stream, "stream"));
 
@@ -95,10 +91,6 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
                         type == TIFFExtension.COMPRESSION_CCITT_T4 ||
                         type == TIFFExtension.COMPRESSION_CCITT_T6,
                 type, "Only CCITT Modified Huffman RLE compression (2), CCITT T4 (3) or CCITT T6 (4) supported: %s");
-        this.fillOrder = Validate.isTrue(
-                fillOrder == TIFFBaseline.FILL_LEFT_TO_RIGHT || fillOrder == TIFFExtension.FILL_RIGHT_TO_LEFT,
-                fillOrder, "Expected fill order 1  or 2: %s"
-        );
 
         // We know this is only used for b/w (1 bit)
         decodedRow = new byte[(columns + 7) / 8];
@@ -140,54 +132,63 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
      * @param columns the number of columns in the stream.
      * @param type the type of stream, must be one of {@code COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE},
      *             {@code COMPRESSION_CCITT_T4} or {@code COMPRESSION_CCITT_T6}.
-     * @param fillOrder fillOrder, must be {@code FILL_LEFT_TO_RIGHT} or
-     * {@code FILL_RIGHT_TO_LEFT}.
      * @param options CCITT T.4 or T.6 options.
      */
-    public CCITTFaxDecoderStream(final InputStream stream, final int columns, final int type, final int fillOrder,
+    public CCITTFaxDecoderStream(final InputStream stream, final int columns, final int type,
                                  final long options) {
-        this(stream, columns, type, fillOrder, options, type == TIFFBaseline.COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE);
+        this(stream, columns, type, options, type == TIFFBaseline.COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE);
     }
 
-    static int findCompressionType(final int type, final InputStream in) throws IOException {
-        // Discover possible incorrect type, revert to RLE
-        if (type == TIFFExtension.COMPRESSION_CCITT_T4 && in.markSupported()) {
-            byte[] streamData = new byte[32];
+    static int findCompressionType(final int encodedType, final InputStream stream) throws IOException {
+        // Discover possible incorrect compression type, revert to RLE if no EOLs found
+        if (encodedType == TIFFExtension.COMPRESSION_CCITT_T4 && stream.markSupported()) {
+            int limit = 512;
 
             try {
-                in.mark(streamData.length);
+                stream.mark(limit);
+                int first = stream.read();
+                int second = stream.read();
 
-                int offset = 0;
-                while (offset < streamData.length) {
-                    int read = in.read(streamData, offset, streamData.length - offset);
-                    if (read <= 0) {
-                        break;
+                if (second == -1) {
+                    // stream to short
+                    return encodedType;
+                }
+                else if (first == 0 && (((byte) second) >> 4 == 1 || ((byte) second) == 1)) {
+                    // correct, starts with EOL or byte aligned EOL
+                    return encodedType;
+                }
+
+                short b = (short) (((((byte) first) << 8) + ((byte) second)) >> 4);
+                int limitBits = limit * 8;
+                int read = second;
+                byte streamByte = (byte) read;
+
+                for (int i = 12; i < limitBits; i++) {
+                    if (i % 8 == 0) {
+                        read = stream.read();
+                        if (read == -1) {
+                            // no EOL before stream end
+                            return TIFFBaseline.COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE;
+                        }
                     }
 
-                    offset += read;
-                }
-            }
-            finally {
-                in.reset();
-            }
-
-            if (streamData[0] != 0 || (streamData[1] >> 4 != 1 && streamData[1] != 1)) {
-                // Leading EOL (0b000000000001) not found, search further and try RLE if not found
-                int numBits = streamData.length * 8;
-                short b = (short) (((streamData[0] << 8) + streamData[1]) >> 4);
-                for (int i = 12; i < numBits; i++) {
-                    b = (short) ((b << 1) + ((streamData[(i / 8)] >> (7 - (i % 8))) & 0x01));
+                    b = (short) ((b << 1) + ((streamByte >> (7 - (i % 8))) & 0x01));
 
                     if ((b & 0xFFF) == 1) {
+                        // found EOL
                         return TIFFExtension.COMPRESSION_CCITT_T4;
                     }
                 }
 
+                // no EOL till limit
                 return TIFFBaseline.COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE;
             }
+            finally {
+                stream.reset();
+            }
         }
 
-        return type;
+        return encodedType;
     }
 
     private void fetch() throws IOException {
@@ -197,13 +198,17 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
             try {
                 decodeRow();
             }
+            catch (ArrayIndexOutOfBoundsException e) {
+                // Mask the AIOOBE as an IOException
+                throw new IOException("Malformed CCITT stream", e);
+            }
             catch (EOFException e) {
                 // TODO: Rewrite to avoid throw/catch for normal flow...
                 if (decodedLength != 0) {
                     throw e;
                 }
 
-                // ..otherwise, just let client code try to read past the
+                // ...otherwise, just let client code try to read past the
                 // end of stream
                 decodedLength = -1;
             }
@@ -468,7 +473,7 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
     int bufferPos = -1;
 
     private boolean readBit() throws IOException {
-        if (bufferPos < 0 || bufferPos > 7) {
+        if (bufferPos > 7 || bufferPos < 0) {
             buffer = in.read();
 
             if (buffer == -1) {
@@ -478,21 +483,10 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
             bufferPos = 0;
         }
 
-        boolean isSet;
-
-        if (fillOrder == TIFFBaseline.FILL_LEFT_TO_RIGHT) {
-            isSet = ((buffer >> (7 - bufferPos)) & 1) == 1;
-        }
-        else {
-            isSet = ((buffer >> (bufferPos)) & 1) == 1;
-        }
-
+        boolean isSet = (buffer & 0x80) != 0;
+        buffer <<= 1;
         bufferPos++;
 
-        if (bufferPos > 7) {
-            bufferPos = -1;
-        }
-
         return isSet;
     }
 
