@@ -32,6 +32,7 @@ package com.twelvemonkeys.imageio.plugins.tiff;
 
 import com.twelvemonkeys.image.ImageUtil;
 import com.twelvemonkeys.imageio.ImageWriterBase;
+import com.twelvemonkeys.imageio.color.ColorProfiles;
 import com.twelvemonkeys.imageio.metadata.Directory;
 import com.twelvemonkeys.imageio.metadata.Entry;
 import com.twelvemonkeys.imageio.metadata.tiff.Rational;
@@ -63,6 +64,7 @@ import java.util.*;
 import java.util.zip.Deflater;
 import java.util.zip.DeflaterOutputStream;
 
+import static com.twelvemonkeys.imageio.plugins.tiff.TIFFImageMetadataFormat.SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME;
 import static com.twelvemonkeys.imageio.plugins.tiff.TIFFStreamMetadata.configureStreamByteOrder;
 
 /**
@@ -202,26 +204,26 @@ public final class TIFFImageWriter extends ImageWriterBase {
             long streamPosition = imageOutput.getStreamPosition();
 
             long ifdSize = tiffWriter.computeIFDSize(entries.values());
-            long stripOffset = streamPosition + 4 + ifdSize + 4;
+            long stripOffset = streamPosition + tiffWriter.offsetSize() + ifdSize + tiffWriter.offsetSize();
             long stripByteCount = ((long) renderedImage.getWidth() * renderedImage.getHeight() * pixelSize + 7L) / 8L;
 
             entries.put(TIFF.TAG_STRIP_OFFSETS, new TIFFEntry(TIFF.TAG_STRIP_OFFSETS, TIFF.TYPE_LONG, stripOffset));
             entries.put(TIFF.TAG_STRIP_BYTE_COUNTS, new TIFFEntry(TIFF.TAG_STRIP_BYTE_COUNTS, TIFF.TYPE_LONG, stripByteCount));
 
-            long ifdPointer = tiffWriter.writeIFD(entries.values(), imageOutput); // NOTE: Writer takes case of ordering tags
+            long ifdPointer = tiffWriter.writeIFD(entries.values(), imageOutput); // NOTE: Writer takes care of ordering tags
             nextIFDPointerOffset = imageOutput.getStreamPosition();
 
             // If we have a previous IFD, update pointer
             if (streamPosition > lastIFDPointerOffset) {
                 imageOutput.seek(lastIFDPointerOffset);
-                imageOutput.writeInt((int) ifdPointer);
+                tiffWriter.writeOffset(imageOutput, ifdPointer);
                 imageOutput.seek(nextIFDPointerOffset);
             }
 
-            imageOutput.writeInt(0); // Update next IFD pointer later
+            tiffWriter.writeOffset(imageOutput, 0); // Update next IFD pointer later
         }
         else {
-            imageOutput.writeInt(0); // Update current IFD pointer later
+            tiffWriter.writeOffset(imageOutput, 0); // Update current IFD pointer later
         }
 
         long stripOffset = imageOutput.getStreamPosition();
@@ -260,7 +262,7 @@ public final class TIFFImageWriter extends ImageWriterBase {
             entries.put(TIFF.TAG_STRIP_OFFSETS, new TIFFEntry(TIFF.TAG_STRIP_OFFSETS, TIFF.TYPE_LONG, stripOffset));
             entries.put(TIFF.TAG_STRIP_BYTE_COUNTS, new TIFFEntry(TIFF.TAG_STRIP_BYTE_COUNTS, TIFF.TYPE_LONG, stripByteCount));
 
-            long ifdPointer = tiffWriter.writeIFD(entries.values(), imageOutput); // NOTE: Writer takes case of ordering tags
+            long ifdPointer = tiffWriter.writeIFD(entries.values(), imageOutput); // NOTE: Writer takes care of ordering tags
 
             nextIFDPointerOffset = imageOutput.getStreamPosition();
 
@@ -268,10 +270,10 @@ public final class TIFFImageWriter extends ImageWriterBase {
             // However, need to update here, because to the writeIFD method writes the pointer, but at the incorrect offset
             // TODO: Refactor writeIFD to take an offset
             imageOutput.seek(lastIFDPointerOffset);
-            imageOutput.writeInt((int) ifdPointer);
+            tiffWriter.writeOffset(imageOutput, ifdPointer);
             imageOutput.seek(nextIFDPointerOffset);
 
-            imageOutput.writeInt(0); // Next IFD pointer updated later
+            tiffWriter.writeOffset(imageOutput, 0); // Next IFD pointer updated later
         }
 
         return nextIFDPointerOffset;
@@ -737,8 +739,8 @@ public final class TIFFImageWriter extends ImageWriterBase {
             TIFFImageMetadata outData = new TIFFImageMetadata(Collections.<Entry>emptySet());
 
             try {
-                if (Arrays.asList(inData.getMetadataFormatNames()).contains(TIFFMedataFormat.SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME)) {
-                    outData.setFromTree(TIFFMedataFormat.SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME, inData.getAsTree(TIFFMedataFormat.SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME));
+                if (Arrays.asList(inData.getMetadataFormatNames()).contains(SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME)) {
+                    outData.setFromTree(SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME, inData.getAsTree(SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME));
                 }
                 else if (inData.isStandardMetadataFormatSupported()) {
                     outData.setFromTree(IIOMetadataFormatImpl.standardMetadataFormatName, inData.getAsTree(IIOMetadataFormatImpl.standardMetadataFormatName));
@@ -768,45 +770,7 @@ public final class TIFFImageWriter extends ImageWriterBase {
         entries.put(TIFF.TAG_SOFTWARE, new TIFFEntry(TIFF.TAG_SOFTWARE, "TwelveMonkeys ImageIO TIFF writer " + originatingProvider.getVersion()));
         entries.put(TIFF.TAG_ORIENTATION, new TIFFEntry(TIFF.TAG_ORIENTATION, 1)); // (optional)
 
-        if (ifd != null) {
-            for (Entry entry : ifd) {
-                int tagId = (Integer) entry.getIdentifier();
-
-                switch (tagId) {
-                    // Baseline
-                    case TIFF.TAG_SUBFILE_TYPE:
-                    case TIFF.TAG_OLD_SUBFILE_TYPE:
-                    case TIFF.TAG_IMAGE_DESCRIPTION:
-                    case TIFF.TAG_MAKE:
-                    case TIFF.TAG_MODEL:
-                    case TIFF.TAG_ORIENTATION:
-                    case TIFF.TAG_X_RESOLUTION:
-                    case TIFF.TAG_Y_RESOLUTION:
-                    case TIFF.TAG_RESOLUTION_UNIT:
-                    case TIFF.TAG_SOFTWARE:
-                    case TIFF.TAG_DATE_TIME:
-                    case TIFF.TAG_ARTIST:
-                    case TIFF.TAG_HOST_COMPUTER:
-                    case TIFF.TAG_COPYRIGHT:
-                    // Extension
-                    case TIFF.TAG_DOCUMENT_NAME:
-                    case TIFF.TAG_PAGE_NAME:
-                    case TIFF.TAG_X_POSITION:
-                    case TIFF.TAG_Y_POSITION:
-                    case TIFF.TAG_PAGE_NUMBER:
-                    case TIFF.TAG_XMP:
-                    // Private/Custom
-                    case TIFF.TAG_IPTC:
-                    case TIFF.TAG_PHOTOSHOP:
-                    case TIFF.TAG_PHOTOSHOP_IMAGE_SOURCE_DATA:
-                    case TIFF.TAG_PHOTOSHOP_ANNOTATIONS:
-                    case TIFF.TAG_EXIF_IFD:
-                    case TIFF.TAG_GPS_IFD:
-                    case TIFF.TAG_INTEROP_IFD:
-                        entries.put(tagId, entry);
-                }
-            }
-        }
+        mergeSafeMetadata(ifd, entries);
 
         ColorModel colorModel = imageType.getColorModel();
         SampleModel sampleModel = imageType.getSampleModel();
@@ -887,9 +851,11 @@ public final class TIFFImageWriter extends ImageWriterBase {
         else {
             entries.put(TIFF.TAG_SAMPLES_PER_PIXEL, new TIFFEntry(TIFF.TAG_SAMPLES_PER_PIXEL, numBands));
 
-            // Note: Assuming sRGB to be the default RGB interpretation
+            // Embed ICC profile if we have one that:
+            // * is not sRGB (assuming sRGB to be the default RGB interpretation), and
+            // * is not gray scale (assuming photometric either BlackIsZero or WhiteIsZero)
             ColorSpace colorSpace = colorModel.getColorSpace();
-            if (colorSpace instanceof ICC_ColorSpace && !colorSpace.isCS_sRGB()) {
+            if (colorSpace instanceof ICC_ColorSpace && !colorSpace.isCS_sRGB() && !ColorProfiles.isCS_GRAY(((ICC_ColorSpace) colorSpace).getProfile())) {
                 entries.put(TIFF.TAG_ICC_PROFILE, new TIFFEntry(TIFF.TAG_ICC_PROFILE, ((ICC_ColorSpace) colorSpace).getProfile().getData()));
             }
         }
@@ -903,6 +869,64 @@ public final class TIFFImageWriter extends ImageWriterBase {
         return new TIFFImageMetadata(entries.values());
     }
 
+    private void mergeSafeMetadata(final Directory ifd, final Map<Integer, Entry> entries) {
+        if (ifd == null) {
+            return;
+        }
+
+        for (Entry entry : ifd) {
+            int tagId = (Integer) entry.getIdentifier();
+
+            switch (tagId) {
+                // Baseline
+                case TIFF.TAG_SUBFILE_TYPE:
+                case TIFF.TAG_OLD_SUBFILE_TYPE:
+                case TIFF.TAG_IMAGE_DESCRIPTION:
+                case TIFF.TAG_MAKE:
+                case TIFF.TAG_MODEL:
+                case TIFF.TAG_ORIENTATION:
+                case TIFF.TAG_X_RESOLUTION:
+                case TIFF.TAG_Y_RESOLUTION:
+                case TIFF.TAG_RESOLUTION_UNIT:
+                case TIFF.TAG_SOFTWARE:
+                case TIFF.TAG_DATE_TIME:
+                case TIFF.TAG_ARTIST:
+                case TIFF.TAG_HOST_COMPUTER:
+                case TIFF.TAG_COPYRIGHT:
+                // Extension
+                case TIFF.TAG_DOCUMENT_NAME:
+                case TIFF.TAG_PAGE_NAME:
+                case TIFF.TAG_X_POSITION:
+                case TIFF.TAG_Y_POSITION:
+                case TIFF.TAG_PAGE_NUMBER:
+                case TIFF.TAG_XMP:
+                // Private/Custom
+                case TIFF.TAG_IPTC:
+                case TIFF.TAG_PHOTOSHOP:
+                case TIFF.TAG_PHOTOSHOP_IMAGE_SOURCE_DATA:
+                case TIFF.TAG_PHOTOSHOP_ANNOTATIONS:
+                case TIFF.TAG_EXIF_IFD:
+                case TIFF.TAG_GPS_IFD:
+                case TIFF.TAG_INTEROP_IFD:
+                    entries.put(tagId, entry);
+                    break;
+                default:
+                    // Allow most extension and custom tags
+                    if (tagId >= 1000 && tagId < 50706) {
+                        entries.put(tagId, entry);
+                    }
+                    // Skip 50 706 - 57 080 (DNG tags)
+                    else if (tagId > 50780 && tagId < 65000) {
+                        entries.put(tagId, entry);
+                    }
+                    // Always allow "the reusable range"
+                    else if (tagId >= 65000 && tagId <= 65535) {
+                        entries.put(tagId, entry);
+                    }
+            }
+        }
+    }
+
     @Override
     public IIOMetadata getDefaultStreamMetadata(final ImageWriteParam param) {
         return super.getDefaultStreamMetadata(param);
@@ -935,11 +959,15 @@ public final class TIFFImageWriter extends ImageWriterBase {
         configureStreamByteOrder(streamMetadata, imageOutput);
 
         writingSequence = true;
-        sequenceTIFFWriter = new TIFFWriter();
+        sequenceTIFFWriter = new TIFFWriter(isBigTIFF() ? 8 : 4);
         sequenceTIFFWriter.writeTIFFHeader(imageOutput);
         sequenceLastIFDPos = imageOutput.getStreamPosition();
     }
 
+    private boolean isBigTIFF() throws IOException {
+        return "bigtiff".equalsIgnoreCase(getFormatName());
+    }
+
     @Override
     public void writeToSequence(final IIOImage image, final ImageWriteParam param) throws IOException {
         if (!writingSequence) {
@@ -995,8 +1023,7 @@ public final class TIFFImageWriter extends ImageWriterBase {
 
         BufferedImage original;
 //        BufferedImage original = ImageIO.read(file);
-        ImageInputStream inputStream = ImageIO.createImageInputStream(file);
-        try {
+        try (ImageInputStream inputStream = ImageIO.createImageInputStream(file)) {
             Iterator<ImageReader> readers = ImageIO.getImageReaders(inputStream);
 
             if (!readers.hasNext()) {
@@ -1026,9 +1053,6 @@ public final class TIFFImageWriter extends ImageWriterBase {
 
             original = reader.read(0, param);
         }
-        finally {
-            inputStream.close();
-        }
 
         System.err.println("original: " + original);
 
@@ -1064,12 +1088,11 @@ public final class TIFFImageWriter extends ImageWriterBase {
 //        output.deleteOnExit();
 
         System.err.println("output: " + output);
-        TIFFImageWriter writer = new TIFFImageWriter(null);
+        TIFFImageWriter writer = new TIFFImageWriter(new TIFFImageWriterSpi());
 //        ImageWriter writer = ImageIO.getImageWritersByFormatName("PNG").next();
 //        ImageWriter writer = ImageIO.getImageWritersByFormatName("BMP").next();
-        ImageOutputStream stream = ImageIO.createImageOutputStream(output);
 
-        try {
+        try (ImageOutputStream stream = ImageIO.createImageOutputStream(output)) {
             writer.setOutput(stream);
 
             ImageWriteParam param = writer.getDefaultWriteParam();
@@ -1087,9 +1110,6 @@ public final class TIFFImageWriter extends ImageWriterBase {
             writer.write(null, new IIOImage(image, null, null), param);
             System.err.println("Write time: " + (System.currentTimeMillis() - start) + " ms");
         }
-        finally {
-            stream.close();
-        }
 
         System.err.println("output.length: " + output.length());
 
